@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* ui.c - main menus and callbacks; utility functions
 
-   Copyright (C) 1998 - 2003 Free Software Foundation
+   Copyright (C) 1998 - 2004 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -29,8 +29,12 @@
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
 
-#include "ghex.h"
+#include "ui.h"
 #include "ghex-window.h"
+#include "findreplace.h"
+#include "converter.h"
+#include "print.h"
+#include "chartable.h"
 
 static void ghex_print(GtkHex *gh, gboolean preview);
 static void ghex_print_run_dialog(GHexPrintJobInfo *pji);
@@ -40,22 +44,38 @@ static void ghex_print_document_real (GHexPrintJobInfo *pji, gboolean preview);
 /* callbacks to nullify widget pointer after a delete event */
 static void about_destroy_cb(GtkObject *, GtkWidget **);
 
-static void open_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void close_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void save_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void save_as_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void print_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void print_preview_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void revert_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void prefs_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void cut_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void copy_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void paste_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void help_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname);
-static void DnDNewWindow_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname);
-static void DnDCancel_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname);
+static void open_cb (BonoboUIComponent *uic, gpointer user_data,
+					 const gchar* verbname);
+static void close_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void save_cb (BonoboUIComponent *uic, gpointer user_data,
+					 const gchar* verbname);
+static void save_as_cb (BonoboUIComponent *uic, gpointer user_data,
+						const gchar* verbname);
+static void print_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void print_preview_cb (BonoboUIComponent *uic, gpointer user_data,
+							  const gchar* verbname);
+static void export_html_cb (BonoboUIComponent *uic, gpointer user_data,
+							const gchar* verbname);
+static void revert_cb (BonoboUIComponent *uic, gpointer user_data,
+					   const gchar* verbname);
+static void prefs_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void about_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void cut_cb (BonoboUIComponent *uic, gpointer user_data,
+					const gchar* verbname);
+static void copy_cb (BonoboUIComponent *uic, gpointer user_data,
+					 const gchar* verbname);
+static void paste_cb (BonoboUIComponent *uic, gpointer user_data,
+					  const gchar* verbname);
+static void help_cb (BonoboUIComponent *uic, gpointer user_data,
+					 const gchar* verbname);
+static void DnDNewWindow_cb(BonoboUIComponent *uic, gpointer user_data,
+							const char *cname);
+static void DnDCancel_cb(BonoboUIComponent *uic, gpointer user_data,
+						 const char *cname);
 
 guint group_type[3] = {
 	GROUP_BYTE,
@@ -145,28 +165,6 @@ delete_event_cb(GtkWidget *w, GdkEventAny *e, GtkWindow *win)
 	return TRUE;
 }
 
-void
-file_sel_ok_cb(GtkWidget *w, gboolean *resp)
-{
-	*resp = TRUE;
-	gtk_main_quit();
-}
-
-void
-file_sel_cancel_cb(GtkWidget *w, gboolean *resp)
-{
-	*resp = FALSE;
-	gtk_main_quit();
-}
-
-gint
-file_sel_delete_event_cb(GtkWidget *w, GdkEventAny *e, gboolean *resp)
-{
-	*resp = FALSE;
-	gtk_main_quit();
-	return TRUE;
-}
-
 gint
 ask_user(GtkMessageDialog *message_box)
 {
@@ -235,15 +233,15 @@ about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	static GtkWidget *about = NULL;
 	
 	static const gchar *authors[] = {
-		"Jaka Mocnik <jaka@gnu.org>",
-		"Chema Celorio <chema@celorio.com>",
-		"Shivram U <shivaram.upadhyayula@wipro.com>",
+		"Jaka Mo\304\215nik",
+		"Chema Celorio",
+		"Shivram Upadhyayula",
 		NULL
 	};
 
 	if(!about) {
 		about = gnome_about_new ( _("GHex, a binary file editor"), VERSION,
-								  "(C) 1998 - 2002 Jaka Mocnik",
+								  "Copyright 1998 - 2004 Jaka Mo\304\215nik",
 								  _("Released under the terms of GNU Public License"),
 								  authors, NULL, NULL, NULL);
 		g_signal_connect(G_OBJECT(about), "destroy",
@@ -286,7 +284,7 @@ paste_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	GHexWindow *win = GHEX_WINDOW(user_data);
 
 	if(win->gh)
-		gtk_hex_paste_clipboard(win->gh);
+		gtk_hex_paste_from_clipboard(win->gh);
 }
 
 void 
@@ -295,7 +293,7 @@ copy_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	GHexWindow *win = GHEX_WINDOW(user_data);
 
 	if(win->gh)
-		gtk_hex_copy_clipboard(win->gh);
+		gtk_hex_copy_to_clipboard(win->gh);
 }
 
 void 
@@ -304,7 +302,7 @@ cut_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 	GHexWindow *win = GHEX_WINDOW(user_data);
 
 	if(win->gh)
-		gtk_hex_cut_clipboard(win->gh);
+		gtk_hex_cut_to_clipboard(win->gh);
 }
 
 void
@@ -365,41 +363,33 @@ open_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
 	GHexWindow *win;
 	GtkWidget *file_sel;
-	gboolean resp;
+	GtkResponseType resp;
 
 	win = GHEX_WINDOW(user_data);
 
-	file_sel = gtk_file_selection_new(NULL);
-
-	gtk_window_set_title(GTK_WINDOW(file_sel), _("Select a file to open"));
-	
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-					  "clicked", G_CALLBACK(file_sel_ok_cb),
-					  &resp);
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-					  "clicked", G_CALLBACK(file_sel_cancel_cb),
-					  &resp);
-	g_signal_connect (G_OBJECT (file_sel),
-					  "delete-event", G_CALLBACK(file_sel_delete_event_cb),
-					  &resp);
-
+	file_sel = gtk_file_chooser_dialog_new(_("Select a file to open"),
+										   GTK_WINDOW(win),
+										   GTK_FILE_CHOOSER_ACTION_OPEN,
+										   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+										   GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+										   NULL);
 	gtk_window_set_modal (GTK_WINDOW(file_sel), TRUE);
 	gtk_window_set_position (GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
 	gtk_widget_show (file_sel);
 
-	gtk_main();
+	resp = gtk_dialog_run(GTK_DIALOG(file_sel));
 
-	if(resp) {
+	if(resp == GTK_RESPONSE_OK) {
 		gchar *flash;
 
 		if(GHEX_WINDOW(win)->gh != NULL) {
-			win = GHEX_WINDOW(ghex_window_new_from_file(gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel))));
+			win = GHEX_WINDOW(ghex_window_new_from_file(gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel))));
 			if(win != NULL)
 				gtk_widget_show(GTK_WIDGET(win));
 		}
 		else {
 			if(!ghex_window_load(GHEX_WINDOW(win),
-								 gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel))))
+								 gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel))))
 				win = NULL;
 		}
 
@@ -419,7 +409,7 @@ open_cb(BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 			display_error_dialog (ghex_window_get_active(), _("Can not open file!"));
 	}
 
-	gtk_widget_destroy(GTK_WIDGET(file_sel));
+	gtk_widget_destroy(file_sel);
 }
 
 static void
@@ -467,7 +457,7 @@ export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbnam
 	GHexWindow *win = GHEX_WINDOW(user_data);
 	HexDocument *doc;
 	GtkWidget *file_sel;
-	gboolean resp;
+	GtkResponseType resp;
 
 	if(win->gh)
 		doc = win->gh->document;
@@ -477,27 +467,23 @@ export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbnam
 	if(doc == NULL)
 		return;
 
-	file_sel = gtk_file_selection_new(NULL);
-
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_sel), doc->file_name);
-	gtk_window_set_title(GTK_WINDOW(file_sel), _("Select path and file name for the HTML source"));	
-	g_signal_connect(G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-					 "clicked", G_CALLBACK(file_sel_ok_cb),
-					 &resp);
-	g_signal_connect(G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-					 "clicked", G_CALLBACK(file_sel_cancel_cb),
-					 &resp);
-	g_signal_connect (G_OBJECT (file_sel),
-					  "delete-event", G_CALLBACK(file_sel_delete_event_cb),
-					  &resp);
+	file_sel = gtk_file_chooser_dialog_new(_("Select path and file name for the HTML source"),
+										   GTK_WINDOW(win),
+										   GTK_FILE_CHOOSER_ACTION_SAVE,
+										   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+										   GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+										   NULL);
+#if 0
+	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(file_sel), doc->file_name);
+#endif
 	gtk_window_set_modal(GTK_WINDOW(file_sel), TRUE);
-	gtk_widget_show(file_sel);
 	gtk_window_set_position(GTK_WINDOW (file_sel), GTK_WIN_POS_MOUSE);
+	gtk_widget_show(file_sel);
 
-	gtk_main();
+	resp = gtk_dialog_run(GTK_DIALOG(file_sel));
 
-	if(resp) {
-		gchar *html_path = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel)));
+	if(resp == GTK_RESPONSE_OK) {
+		gchar *html_path = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_sel)));
 		gchar *sep, *base_name, *check_path;
 		GtkHex *view = win->gh;
 
@@ -541,6 +527,7 @@ export_html_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbnam
 											"Do you want to proceed?"));
 			gtk_dialog_set_default_response(GTK_DIALOG(mbox), GTK_RESPONSE_NO);
 			reply = ask_user(GTK_MESSAGE_DIALOG(mbox));
+			gtk_widget_destroy(mbox);
 			if(reply != GTK_RESPONSE_YES) {
 				g_free(html_path);
 				g_free(check_path);
@@ -610,6 +597,10 @@ close_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 
     /* this implicitly destroys all views including this one */
     g_object_unref(G_OBJECT(doc));
+
+    /* This is to clear the contents of status  bar after closing the files */
+    bonobo_ui_component_set_status(win->uic, " ", NULL);
+
 }
 
 void
@@ -649,7 +640,7 @@ void prefs_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname
 	if(!prefs_ui)
 		prefs_ui = create_prefs_dialog();
 
-	set_prefs(prefs_ui);
+	set_current_prefs(prefs_ui);
 
 	if(ghex_window_get_active() != NULL)
 		gtk_window_set_transient_for(GTK_WINDOW(prefs_ui->pbox),
@@ -705,6 +696,8 @@ revert_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 			g_free(gtk_file_name);
 			g_free(flash);
 		}
+
+		gtk_widget_destroy (mbox);
 	}
 }
 
@@ -741,9 +734,32 @@ ghex_print(GtkHex *gh, gboolean preview)
 	}
 }
 
+static void
+ghex_print_progress(gint page, gint ofpages, gpointer data)
+{
+	GtkProgressBar *progress_bar = GTK_PROGRESS_BAR(data);
+	gchar *progress_str;
+
+	gtk_progress_bar_set_fraction(progress_bar,
+								  (gdouble)page/(gdouble)ofpages);
+	progress_str = g_strdup_printf("%d/%d", page, ofpages);
+	gtk_progress_bar_set_text(progress_bar, progress_str);	
+	g_free(progress_str);
+	while(gtk_events_pending())
+		gtk_main_iteration();
+}
+
+static gboolean
+ignore_cb(GtkWidget *w, GdkEventAny *e, gpointer user_data)
+{
+	return TRUE;
+}
+
 static void
 ghex_print_document_real(GHexPrintJobInfo *pji, gboolean preview)
 {
+	GtkWidget *progress_dialog, *progress_bar;
+
 	g_return_if_fail (pji->config != NULL);
 
 	pji->master = gnome_print_job_new (pji->config);
@@ -751,7 +767,23 @@ ghex_print_document_real(GHexPrintJobInfo *pji, gboolean preview)
 	g_return_if_fail (pji->master != NULL);
 
 	ghex_print_update_page_size_and_margins (pji->doc, pji);
-	ghex_print_job_execute(pji);
+
+	progress_dialog = gtk_dialog_new();
+	gtk_window_set_resizable(GTK_WINDOW(progress_dialog), FALSE);
+	gtk_window_set_modal(GTK_WINDOW(progress_dialog), TRUE);
+	g_signal_connect(G_OBJECT(progress_dialog), "delete-event",
+					 G_CALLBACK(ignore_cb), NULL);
+	gtk_window_set_title(GTK_WINDOW(progress_dialog),
+						 _("Printing file..."));
+	progress_bar = gtk_progress_bar_new();
+	gtk_widget_show(progress_bar);
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(progress_dialog)->vbox),
+					  progress_bar);
+	gtk_widget_show(progress_dialog);
+
+	ghex_print_job_execute(pji, ghex_print_progress, progress_bar);
+
+	gtk_widget_destroy(progress_dialog);
 
 	if(pji->preview)
 		ghex_print_preview_real(pji);
@@ -777,11 +809,6 @@ ghex_print_dialog_response (GtkWidget *dialog, int response,
 		return;
 	}
 
-	if (pji->config == NULL) {
-		pji->config = gnome_print_dialog_get_config
-			(GNOME_PRINT_DIALOG(dialog));		
-	}
-	
 	pji->range = gnome_print_dialog_get_range_page
 		(GNOME_PRINT_DIALOG(dialog), &pji->page_first, &pji->page_last);
 	
@@ -810,6 +837,13 @@ ghex_print_run_dialog(GHexPrintJobInfo *pji)
 			     (const char *) _("Print Hex Document"),
 			     GNOME_PRINT_DIALOG_RANGE);
 
+	if (pji->config == NULL) {
+		pji->config = gnome_print_dialog_get_config(
+						GNOME_PRINT_DIALOG(dialog));
+	}
+
+	ghex_print_update_page_size_and_margins (pji->doc, pji);
+
 	gnome_print_dialog_construct_range_page((GnomePrintDialog *)dialog,
 			GNOME_PRINT_RANGE_ALL | GNOME_PRINT_RANGE_RANGE,
 			1, pji->pages, "A", _("Pages"));
